package djmil.hellomvc;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.security.provisioning.InMemoryUserDetailsManager;
import org.springframework.security.web.SecurityFilterChain;

@Configuration
public class SecurityConfig {

    @Bean
    public SecurityFilterChain filterChain(HttpSecurity http) throws Exception {

        http
        .authorizeHttpRequests(authz -> authz      
            .requestMatchers("/secret").authenticated()
            //.hasRole("CARD-OWNER") // <<-- enable RBAC
            .anyRequest().permitAll()
        );

        http
		.formLogin(form -> form
			.loginPage("/login")
			.permitAll()
		);

        return http.build();
    }

    @Bean
    public PasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }

    @Bean
    public UserDetailsService testOnlyUsers(PasswordEncoder passwordEncoder) {
        User.UserBuilder users = User.builder();
	
        UserDetails sarah = users
            .username("agent")
            .password(passwordEncoder.encode("qaz123"))
            //.roles("CARD-OWNER") // no roles for now
            .build();
        
        return new InMemoryUserDetailsManager(sarah);
    }
}
